@@ -5,7 +5,7 @@
 $::PROTOCOL = '(?:s?https?|ftp)';
 $::RXURL    = '(?:s?https?|ftp)://[-\\w.!~*\'();/?:@&=+$,%#]+' ;
 $::charset  = 'EUC-JP';
-$::version  = '1.1.3_0 ($Date: 2006/08/12 05:28:04 $)';
+$::version  = '1.1.5_0 ($Date: 2006/10/09 05:10:19 $)';
 %::form     = ();
 $::me       = $::postme = $ENV{SCRIPT_NAME};
 $::print    = ' 'x 10000; $::print = '';
@@ -97,13 +97,14 @@ sub init_globals{
         'v'        => sub{ '&' . ($#_ >= 1 ? $_[1] : 'amp') . ';' },
         'bq'       => sub{ '&#96;' },
         'null'     => sub{ '' } ,
+        'outline'  => \&plugin_outline ,
     );
 
     %::action_plugin = (
-        'index'         => sub{ &do_index('recent','rindex','-l');  },
-        'rindex'        => sub{ &do_index('recent','index' ,'-l','-r'); },
-        'older'         => sub{ &do_index('recent','index' ,'-l','-t'); },
-        'recent'        => sub{ &do_index('older' ,'index' ,'-l','-t','-r');},
+        'index'         => sub{ &do_index('recent','rindex','-i','-a','-l');  },
+        'rindex'        => sub{ &do_index('recent','index' ,'-i','-a','-l','-r'); },
+        'older'         => sub{ &do_index('recent','index' ,'-i','-a','-l','-t'); },
+        'recent'        => sub{ &do_index('older' ,'index' ,'-i','-a','-l','-t','-r');},
         '?'             => \&action_seek ,
         'edt'           => \&action_edit ,
         'pwd'           => \&action_passwd ,
@@ -130,7 +131,7 @@ sub init_globals{
         <link rel="index" href="$::me?a=index">)
     );
 
-    @::body_header = <DATA>;
+    @::body_header = ( $::config{body_header}||'' );
 
     @::menubar = (
         &anchor( $::config{FrontPage} , undef  ) ,
@@ -167,20 +168,22 @@ sub init_globals{
 
     %::preferences = (
         ' General Options' => [
-            { desc=>'the sitename', name=>'sitename', size=>40 },
-            { desc=>'enable link to file://...', name=>'locallink' ,
+            { desc=>'The sitename', name=>'sitename', size=>40 },
+            { desc=>'Enable link to file://...', name=>'locallink' ,
               type=>'checkbox' },
-            { desc=>'forbid any one but administrator creating a new page.' ,
+            { desc=>'Forbid any one but administrator creating a new page.' ,
               name=>'lonely' , type=>'checkbox' },
-            { desc=>'target value for external link.',name=>'target'},
-            { desc=>'pagename(s) for CSS (1-line for 1-page)' ,
-              name=>'CSS' , type=>'textarea' , rows=>1 },
-            { desc=>'pagename for FrontPage'  , name=>'FrontPage' , size=>40 },
+            { desc=>'Target value for external link.',name=>'target'},
+            { desc=>'Pagename(s) for CSS (1-line for 1-page)' ,
+              name=>'CSS' , type=>'textarea' , rows=>2 },
+            { desc=>'Pagename for FrontPage'  , name=>'FrontPage' , size=>40 },
+            { desc=>'HTML-Code after <body> (for banner)' ,
+              name=>'body_header' , type=>'textarea' , rows=>2 },
         ],
         ' Section Marks' => [
-            { desc=>'section mark', name=>'sectionmark', size=>3 } ,
-            { desc=>'subsection mark' , name=>'subsectionmark' , size=>3 } ,
-            { desc=>'subsubsection mark' , name=>'subsubsectionmark' , size=>3 }
+            { desc=>'Section mark', name=>'sectionmark', size=>3 } ,
+            { desc=>'Subsection mark' , name=>'subsectionmark' , size=>3 } ,
+            { desc=>'Subsubsection mark' , name=>'subsubsectionmark' , size=>3 }
         ]
     );
 
@@ -208,7 +211,10 @@ sub init_globals{
         '900_seperator'  => \&block_separator ,
         '990_normal'     => \&block_normal ,
     );
-    @::footer_plugin = ( \&plugin_footnote_flush );
+    @::footer_plugin    = ( \&plugin_footnote_flush );
+    @::atodekaku_plugin = ( \&atodekaku_outline );
+
+    @::outline = ();
 }
 
 sub read_multimedia{
@@ -267,6 +273,7 @@ sub putenc{
 }
 
 sub flush{
+    grep( $_->( \$main::print ) , @::atodekaku_plugin );
     $::print =~ s|\a((?:[0-9a-f][0-9a-f])*)\a|pack('h*',$1)|ges;
     print $::print;
 }
@@ -377,20 +384,19 @@ sub is{ $::config{$_[0]} && $::config{$_[0]} ne 'NG' ; }
 sub print_form{
     my ($title,$html,$stamp) = @_;
 
-    &putenc('<div class="update"
-        ><form name="editform" action="%s" 
-         enctype="multipart/form-data" method="post"
-        ><input type="hidden" name="stamp" value="%s"
+    &putenc('<div class="update"><form name="editform" action="%s"
+          enctype="multipart/form-data" method="post"
+          accept-charset="%s" ><input type="hidden" name="stamp" value="%s"
         ><input type="hidden" name="p" value="%s"><br>'
-        , $::postme,$stamp,$title );
+        , $::postme , $::charset , $stamp , $title );
     &puts('<textarea cols="80" rows="20" name="honbun">'.$html.
             '</textarea><br>');
 
     my $fname=&title2fname( $::form{p} );
     &puts('<input type="submit" name="a" value="Preview">');
     if( exists $::form{admin} || &is_frozen() ){
-        &puts('Sign: <input type="password" name="password"
-        ><input type="checkbox" name="to_freeze" value="1"');
+        &print_signarea();
+        &puts('<input type="checkbox" name="to_freeze" value="1"');
         &is_frozen() and &puts('checked');
         &puts(' >freeze <input type="hidden" name="admin" value="admin">');
     }
@@ -399,7 +405,7 @@ sub print_form{
     &puts('<h2>Attachment</h2>
     <p>New:<input type="file" name="butsu" size="48">');
     if( exists $::form{admin} || &is_frozen() ){
-        &puts('Sign:<input type="password" name="qassword">');
+        &print_signarea('qassword');
     }
     &puts('<input type="submit" name="a" value="Upload"></p>');
     
@@ -450,6 +456,7 @@ sub print_header{
     &putenc('<title>%s</title><style type="text/css"><!--',$label);
     foreach my $p (split(/\s*\n\s*/,$::config{CSS})){
         if( my $css =&read_object($p) ){
+            $css =~ s/\<\<\{([^\}]+)\}/&myurl( { p=>$p , f=>$1 } )/ge;
             $css =~ s/[<>&]//g;
             $css =~ s|/\*.*?\*/||gs;
             &puts( $css );
@@ -499,6 +506,10 @@ sub ninsho{
     }
 }
 
+sub print_signarea{
+    &putenc('Sign: <input type="password" name="%s">',$_[0] || 'password');
+}
+
 sub check_frozen{
     if( exists $::form{admin} ){ ### Administrator mode ###
          &ninsho;
@@ -555,11 +566,12 @@ sub write_file{
 
 sub action_new{
     &print_header( divclass=>'max' );
-    &puts(qq(<h1>Create Page</h1>
-        <form action="$::postme" method="post">
+    &putenc(qq(<h1>Create Page</h1>
+        <form action="%s" method="post" accept-charset="%s">
         <p><input type="text" name="p" size="40">
         <input type="hidden" name="a" value="edt">
-        <input type="submit" value="Create"></p></form>));
+        <input type="submit" value="Create"></p></form>)
+        , $::postme , $::charset );
     &print_footer;
 }
 
@@ -583,9 +595,8 @@ sub action_query_delete{
     &putenc( q( <h1>Remove attachment</h1><p>
                 Remove attachment '%s' of '%s'.<br>)
             , $::form{f} , $::form{p} );
-    &is_frozen() and &puts( q(Administrator's Sign:
-            <input type="password" name="password"><br>) );
-    &putenc('Are you sure ? <br>
+    &is_frozen() and &print_signarea();
+    &putenc('<div>Are you sure ? </div>
         <input type="submit" name="yes" value="Yes">
         <input type="submit" name="no" value="No">
         <input type="hidden" name="a" value="del">
@@ -687,6 +698,10 @@ sub action_tools{
                           ? ' checked' : '' )
                         , $p->[1] );
                 }
+            }elsif( $i->{type} eq 'a' ){
+                &putenc('<a href="%s">%s</a><br>',$i->{href},$i->{desc} );
+            }elsif( $i->{type} eq 'rem' ){
+                &putenc('%s<br>',$i->{desc} );
             }else{ # text
                 &putenc(
                     '%s <input type="text" name="%s" value="%s" size="%s"><br>'
@@ -698,9 +713,9 @@ sub action_tools{
         }
         &puts('</p></div></div>');
     }
-    &puts('Sign: <input type="password" name="password"
-        ><input type="hidden" name="a" value="preferences"
-        ><input type="submit" value="Submit"></form></div></div>');
+    &print_signarea();
+    &puts('<input type="hidden" name="a" value="preferences">',
+          '<input type="submit" value="Submit"></form></div></div>');
 
     &print_footer;
 }
@@ -889,11 +904,10 @@ sub action_edit{
             <p><form action="%s" method="post">
             <input type="hidden"  name="a" value="ren">
             <input type="hidden"  name="p" value="%s">
-            Title: <input type="text" name="newtitle" value="%s" size="80">
-            <br>Sign: <input type="password" name="password">
-            <br><input type="submit" name="ren" value="Submit">
-            </form></p>
-        ' , $::postme , $::form{p} , $::form{p} );
+            Title: <input type="text" name="newtitle" value="%s" size="80">'
+            , $::postme , $::form{p} , $::form{p} );
+        &print_signarea();
+        &puts('<br><input type="submit" name="ren" value="Submit"></form></p>');
     }
     &end_day();
     &print_footer;
@@ -961,9 +975,9 @@ sub object_exists{
 
 sub list_attachment{
     &cache_update();
-    map(  &fname2title(substr($_,2))
-        , grep(defined($_) && /^__/,
-               @{ $::dir_cache{&title2fname( shift ) } } ) );
+    map{ &fname2title(substr($_,2)) }
+        grep{ defined($_) && /^__/ }
+            @{ $::dir_cache{ &title2fname( shift ) } };
 }
 
 sub print_page{
@@ -972,6 +986,10 @@ sub print_page{
     my $html =( exists $args{html} ? $args{html} : &enc( &read_object($title)) );
     return 0 unless $html;
 
+    push(@main::outline,
+        { depth => -1 , text  => $title , title => $title , sharp => '' }
+    );
+
     my %attachment;
     foreach my $attach ( &list_attachment($title) ){
         my $e_attach = &enc( $attach );
@@ -1077,6 +1095,39 @@ sub plugin_footnote_flush{
     delete $session->{footnotes};
 }
 
+sub plugin_outline{
+    "\x1B(outline)";
+}
+
+sub atodekaku_outline{
+    my ($print)=@_;
+    my $depth=-2;
+    my $ss='';
+    foreach my $p( @::outline ){
+        next if $p->{title} eq 'Header' || 
+                $p->{title} eq 'Footer' || 
+                $p->{title} eq 'Sidebar' ;
+
+        my $diff=$p->{depth} - $depth;
+        if( $diff > 0 ){
+            $ss .= '<ul><li>' x $diff ;
+        }else{
+            $diff < 0    and $ss .= "</li></ul>\n" x -$diff;
+            $depth >= 0  and $ss .= "</li>\n" ;
+            $ss .= '<li>';
+        }
+        $ss .= &anchor( 
+            $p->{text} ,
+            { p=>$p->{title} } ,
+            undef ,
+            $p->{sharp} );
+        $depth=$p->{depth};
+    }
+    $ss .= '</li></ul>' x ($depth-2);
+
+    $$print =~ s/\x1B\(outline\)/$ss/g;
+}
+
 sub ls_core{
     my $opt = shift;
     my @list;
@@ -1088,10 +1139,18 @@ sub ls_core{
         $pat =~ s/\?/../g;
         $pat =~ s/\*/.*/g;
         $pat = '^' . $pat . '$';
-        push(@list, map({ fname  => $_ ,
-                          title  => &fname2title($_) ,
-                          mtime  => &mtime($_)
-                        } , grep($_ =~ $pat, &list_page() ) ) );
+        push(@list, map{
+             +{ fname  => $_ ,
+                title  => &fname2title($_) ,
+                mtime  => &mtime($_)
+              }
+            }grep{
+                  exists $opt->{a} 
+                ? ($_ =~ $pat )
+                : ($_ =~ $pat && !/^e2/ && -f $_ )
+            }
+            &list_page()
+        );
     }
     if( exists $opt->{t} ){
         @list = sort{ $a->{mtime} cmp $b->{mtime} } @list;
@@ -1128,6 +1187,8 @@ sub ls{
     foreach my $p ( &ls_core(\%opt,@arg) ){
         $buf .= '<li>';
         exists $opt{l} and $buf .= '<tt>'.$p->{mtime}.' </tt>';
+        exists $opt{i} and $buf .= '<tt>['.scalar(@{$::dir_cache{ $p->{fname} }}).'] </tt>';
+
         $buf .= &anchor( &enc($p->{title}) , { p=>$p->{title} } );
         $buf .= "</li>\r\n";
     }
@@ -1163,20 +1224,22 @@ sub plugin_comment{
     }
     unless( exists $opt{f} ){
         $buf .= <<HTML
+<div class="form">
 <form action="$::postme" method="post" class="comment">
 <input type="hidden" name="p" value="$etitle">
 <input type="hidden" name="a" value="comment">
 <input type="hidden" name="comid" value="$ecomid">
 <div class="field name">
 <input type="text" name="who" size="10" class="field">
-</div>
+</div><!-- div.field name -->
 <div class="textarea">
 <textarea name="comment" cols="60" rows="1" class="field"></textarea>
-</div>
+</div><!-- div.textarea -->
 <div class="button">
 <input type="submit" name="Comment" value="Comment">
-</div>
+</div><!-- div.button -->
 </form>
+</div><!-- div.form -->
 HTML
     }
     $buf . '</div></div>';
@@ -1291,8 +1354,19 @@ sub midashi{
         my $h    = $depth+ 3 ;
         my $cls  = ('sub' x $depth).'section' ;
 
+        push( @main::outline ,
+                {
+                  depth => $depth ,
+                  text  => $text ,
+                  title => $session->{title} ,
+                  sharp => "#p$tag"
+                }
+        );
+
         $text =~ s/^\+/${tag}. /;
-        $text = &anchor( &enc($::config{"${cls}mark"})
+        $text = &anchor( '<span class="sanchor">' .
+                         &enc($::config{"${cls}mark"}) .
+                         '</span>' 
                   , { p     => $session->{title} }
                   , { class => "${cls}mark sanchor" }
                   , "#p${tag}"
@@ -1451,5 +1525,3 @@ sub block_normal{
     }
     1;
 }
-__END__
-<!-- Generated with wifky (c) 2005-2006 HAYAMA_Kaoru. -->
